@@ -23,34 +23,29 @@ class qhtml:
         self.scripts_on_page_load = []
         self.head = []
         self.css = self._css()
-        self.display = self.new("div", self)
+        self.display = self.new("div")
         self.bootstrap = self.bootstrap()
         self.preview = self.new("div")
         self.last = None
         # why do i have to do this self self
         self.templates = self._templates(self)
+        self.seo = self.__seo(self)
 
-    # Returns a new object of an html element
+    # Returns a new q_element object
     # returns: Object
 
-    def new(self, _type, _p=0):
+    def new(self, _type):
         _split = _type.split(" ")
         _obj = ""
         if len(_split) > 1:
             _first = _split[0]
             _second = _split[1]
-
             if "button" in _first or "input" in _first:
                 if _second == "br":
-                    _container = self.new_obj("div")
-                    _br = self.new_obj("br")
-                    if _p != 0:
-                        _obj = self.new_obj(_first, _p)
-                        _container.insert([_obj, _br])
-                    else:
-                        _obj = self.new_obj(_first)
-                        _container.insert([_obj, _br])
-
+                    _container = self._q_element("div")
+                    _br = self._q_element("br")
+                    _obj = self._q_element(_first)
+                    _container.insert([_obj, _br])
                     _obj.parent = self
                     self.last = _obj
                     self.all.append(_container)
@@ -62,24 +57,22 @@ class qhtml:
             else:
                 return False
         else:
-            if _p != 0:
-                # create special
-                _obj = self.new_obj(_type, _p)
-            else:
-                _obj = self.new_obj(_type)
-
+            _obj = self._q_element(_type)
             _obj.parent = self
             self.all.append(_obj)
             self.last = _obj
             return _obj
 
+    # Duplicates an element
+    # returns: new duped element
+
     def dupe(self, qhtml_obj):
-        if isinstance(qhtml_obj, self.new_obj):
+        if isinstance(qhtml_obj, self._q_element):
             new = copy.copy(qhtml_obj)
             self.all.append(new)
             return new
         else:
-            print('New obj instance is not valid or was not provided.')
+            print('q_element instance is not valid or was not provided.')
             return False
 
     # Attempts to render the constructed webpage
@@ -96,14 +89,11 @@ class qhtml:
         _path = 'C:/Program Files (x86)/Google/Chrome/Application/chrome.exe %s'
         _preview_scripts_loaded = False
         for _obj_element in self.all:
-            # print("ajax code for " + str(_obj_element.type))
-
             if _obj_element.has_preview() and _preview_scripts_loaded is False:
                 self.__get_preview_scripts()
                 _preview_scripts_loaded = True
 
             if _obj_element.ajax_code != "":
-                # print("render ajax code detected")
                 self.scripts.append(_obj_element.ajax_code)
 
             if len(_obj_element.scripts_on_page_load) > 0:
@@ -129,19 +119,22 @@ class qhtml:
         for h in self.head:
             _head_append += h + "\n"
 
-        print("inserting " + str(self.preview))
-        self.display.insert(self.preview)
-        _scripts_on_page_load = ' window.addEventListener("load", on_page_load_init); function on_page_load_init() {' + _scripts_on_page_load + '}'
-        html_string = "<head>" + _bootstrap + "<style>" + _css + '</style><script type="text/javascript">' + _scripts + '' + _scripts_on_page_load + '</script>' + _head_append + '</head>' + str(
-            self.all[0].innerHTML)
+        if _preview_scripts_loaded:
+            self.display.insert(self.preview)
+
+        if _scripts_on_page_load != "":
+            _scripts_on_page_load = ' window.addEventListener("load", on_page_load_init); function on_page_load_init() {' + _scripts_on_page_load + '}'
+
+        if _scripts != "" or _scripts_on_page_load != "":
+            _scripts = '<script type="text/javascript">' + _scripts + '' + _scripts_on_page_load + '</script>'
+
+        html_string = "<head>" + _head_append + _bootstrap + "<style>" + _css + '</style>' + _scripts + '</head>' + str(self.all[0].innerHTML)
 
         f = open(os.getcwd() + "/" + output_file, "w")
         f.write(html_string)
         f.close()
-
-        # print(html_string)
-
         sleep(0.2)
+
         if not only_html:
             webbrowser.get(_path).open(str(os.getcwd()) + "/" + output_file)
 
@@ -150,17 +143,25 @@ class qhtml:
 
         return html_string
 
+    # Attempts to put a string to the users clipboard
+    # returns: void
+
     def clip_put(self, _str):
         s = self
-        # Check which operating system is running to get the correct copying keyword.
+        copy_keyword = ""
         if platform.system() == 'Darwin':
             copy_keyword = 'pbcopy'
         elif platform.system() == 'Windows':
             copy_keyword = 'clip'
-
+        else:
+            print('Could not copy to clipboard for some reason.')
+            return False
         subprocess.run(copy_keyword, universal_newlines=True, input=_str)
 
-    def file_read(self,file_name,file_path=''):
+    # Easily read a file. Returns the file's contents as a string
+    # returns: file-contents/boolean
+
+    def file_read(self, file_name, file_path='', to_list=False):
         s = self
         if file_name:
             if file_path:
@@ -170,9 +171,9 @@ class qhtml:
             f = open(dir_path + file_name, 'r')
             read = f.read()
             f.close()
-
+            if to_list:
+                read = read.split('\n')
             return read
-
         return False
 
     def __get_preview_scripts(self):
@@ -252,7 +253,6 @@ class qhtml:
         # returns: class object/itself
 
         def add(self, name: str or list, style=""):
-
             if style == "":
                 if type(name) is list:
                     for li in name:
@@ -279,7 +279,6 @@ class qhtml:
             _b = ""
             for s in self.styles:
                 _b = _b + "" + s + "\n"
-            print("b = " + _b)
             f.write(_b)
             f.close()
 
@@ -297,6 +296,9 @@ class qhtml:
                 self.DARK_GREEN = "#21573a"
                 self.DARK_BROWN = "#403735"
 
+                self.WHITE = "#FFFFFF"
+                self.BLACK = "#000000"
+
         class _helpers:
             def __init__(self):
                 pass
@@ -338,9 +340,9 @@ class qhtml:
                 _hex = _hex.lstrip('#')
                 return tuple(int(_hex[i:i + 2], 16) for i in (0, 2, 4))
 
-    # CLASS new_obj, an element object type
+    # CLASS _q_element, an element object type
 
-    class new_obj:
+    class _q_element:
 
         # INITIALIZE CLASS
 
@@ -379,6 +381,9 @@ class qhtml:
                     else:
                         self.parent.render(output_file)
 
+        # Adds javascript code to the main page/display
+        # returns: self
+
         def scripts_add(self, js_code, on_page_load=False):
             if on_page_load:
                 self.scripts_on_page_load.append(js_code)
@@ -427,6 +432,7 @@ class qhtml:
 
         # Adds an attribute to an element
         # returns: self/object
+
         def add_attribute(self, _str):
             if len(_str) < 4:
                 print('QuykHtml add_attribute error, no value defined!\n- > ' + _str)
@@ -441,6 +447,9 @@ class qhtml:
             self.attributes.append(_attr_name + "=" + _attr_val)
             return self
 
+        # Attempts to clear an attribute from an element
+        # returns: self
+
         def clear_attribute(self, _attr_name):
             if _attr_name in self.attr_check:
                 for _a in self.attributes:
@@ -470,7 +479,6 @@ class qhtml:
                 second = ' style="' + self.style.get() + '">'
             else:
                 second = ">"
-            # return "<" + self.type + " " + self.get_attributes() + ' style="' + self.style.get() + '">'
             return first + second
 
         # Gets the closing tag of an object as a string
@@ -479,6 +487,9 @@ class qhtml:
         def get_tag_close(self):
             return "</" + self.type + ">"
 
+        # Sets a code block to display code
+        # returns: self
+
         def set_text_code_block(self, _str, text_color=False, parentheses_color=False, main_text_color=False, background_color=False):
 
             if self.type != 'pre':
@@ -520,15 +531,24 @@ class qhtml:
             self.innerText = _str
             return self
 
+        # Attempts to set a MEDIUM sized text block used as a place holder
+        # returns: self
+
         def set_text_ipsum(self):
             self.innerText = "Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed do eiusmod tempor incididunt ut labore et dolore magna aliqua. Ut enim ad minim veniam, quis nostrud exercitation ullamco laboris nisi ut aliquip ex ea commodo consequat. Duis aute irure dolor in " \
                              "reprehenderit in voluptate velit esse cillum dolore eu fugiat nulla pariatur. Excepteur sint occaecat cupidatat non proident, sunt in culpa qui officia deserunt mollit anim id est laborum "
             return self
 
+        # Attempts to set a SMALL sized text block used as a place holder
+        # returns: self
+
         def set_text_ipsum_small(self):
             self.innerText = "Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed do eiusmod tempor incididunt ut labore et dolore magna aliqua. Ut enim ad minim veniam, quis nostrud exercitation ullamco laboris nisi ut aliquip ex ea commodo consequat."
             return self
 
+        # Attempts to set a LARGE sized text block used as a place holder
+        # returns: self
+
         def set_text_ipsum_large(self):
             self.innerText = "Sed ut perspiciatis unde omnis iste natus error sit voluptatem accusantium doloremque laudantium, totam rem aperiam, eaque ipsa quae ab illo inventore veritatis et quasi architecto beatae vitae dicta sunt explicabo. Nemo enim ipsam voluptatem quia voluptas sit " \
                              "aspernatur aut odit aut fugit, sed quia consequuntur magni dolores eos qui ratione voluptatem sequi nesciunt. Neque porro quisquam est, qui dolorem ipsum quia dolor sit amet, consectetur, adipisci velit, sed quia non numquam eius modi tempora incidunt ut labore et " \
@@ -544,32 +564,73 @@ class qhtml:
             self.add_attribute('class="' + str(_str) + '"')
             return self
 
+        # Sets an elements id
+        # returns: self
+
         def set_id(self, _str):
             self.add_attribute('id="' + str(_str).replace("#", "") + '"')
             self.id = str(_str).replace("#", "")
             return self
 
+        # Sets an images source (external url or local url)
+        # returns: self
+
         def set_img_src(self, _str):
             self.add_attribute('src="' + _str + '"')
             return self
 
+        # Sets an images alt text
+        # returns: self
+
+        def set_img_alt(self, _str):
+            if self.type != 'img':
+                print('q_element.set_img_alt ERROR.\nset_img_alt should be used on an img type, it was used on a ' + self.type)
+            if _str:
+                self.add_attribute('alt="' + _str + '"')
+
+        # Sets an image placeholder. You can specify
+        # a size by providing an int.
+        # Shout out to via.placeholder.com
+        # returns: self
+
+        def set_img_placeholder(self, place_holder_size=150):
+            if self.type != 'img':
+                print('q_element.set_img_placeholder ERROR.\nShould be used on img type, you used it on ' + self.type)
+                return False
+            self.set_img_src('https://via.placeholder.com/' + str(place_holder_size))
+            return self
+
+        # Sets an elements name attribute
+        # returns: self
+
         def set_name(self, _str):
             self.add_attribute('name="' + _str + '"')
             return self
 
+        # Attempts to set an elements value
+        # returns: self
+
         def set_value(self, _str):
             self.add_attribute('value="' + _str + '"')
             return self
 
+        # Sets an elements tooltip (on mouse hover -> text shows near cursor)
+        # returns: self
+
         def set_tool_tip(self, _str):
             self.add_attribute('title="' + _str + '"')
             return self
 
+        # Sets the element and all of it's children's html into the users' clipboard
+        # returns: self
+
         def set_clip_board(self):
             self.parent.clip_put(self.html())
-            # pyperclip.copy(self.html())
             return self
 
+        # On click, go to specified url. no_https=True will allow for external domain linking.
+        # returns: self
+
         def on_click_goto(self, url_to_nav_to, new_tab=True, no_https=False):
             if no_https is False:
                 if new_tab:
@@ -583,7 +644,10 @@ class qhtml:
                     self.on_click("window.location.href='" + url_to_nav_to + "';")
             return self
 
+        # Sets a form elements form options
         # action="upload.php" method="post" enctype="multipart/form-data"
+        # returns: self
+
         def set_form_options(self, action_php_call, method_get_or_post, enctype="multipart/form-data"):
             method = method_get_or_post
             action = action_php_call
@@ -599,10 +663,17 @@ class qhtml:
                 return self
             return self
 
+        # Makes the submit button inside of a specified URL element a form button
+        # which will submit the form
+        # returns: self
+
         def set_form_button(self):
             self.add_attribute('value="submit"')
             return self
 
+        # Sets an iframe element options
+        # returns: self
+
         def set_iframe(self, src_url, title="iframe"):
             if self.type != "iframe":
                 print("Cannot set iframe data on type " + self.type)
@@ -611,6 +682,9 @@ class qhtml:
             self.add_attribute('src="' + src_url + '" title="' + title + '"')
             return self
 
+        # Makes element (input) auto complete text or not
+        # returns: self
+
         def set_auto_complete(self, _boolean: bool):
             if not _boolean:
                 self.add_attribute('autocomplete="off"')
@@ -619,7 +693,10 @@ class qhtml:
 
             return self
 
-        def html(self,set_clip_board=False):
+        # Gets all of the html of an element (and innerHtml)
+        # returns: self
+
+        def html(self, set_clip_board=False):
             html = self.get_tag_open() + self.innerText + self.innerHTML + self.get_tag_close()
             if set_clip_board:
                 self.parent.clip_put(html)
@@ -671,6 +748,9 @@ class qhtml:
                 self.add_attribute('onClick="' + _code + '"')
             return self
 
+        # On right click, run specified code
+        # returns: self
+
         def on_right_click(self, _code, _block_normal_context=True):
             _block = ''
 
@@ -682,6 +762,10 @@ class qhtml:
             self.add_attribute('oncontextmenu="' + _code + _block + '"')
             return self
 
+        # Upon element clicked, shows a preview full-screen'd image of the element
+        # This can be used for any element, but makes most sense for images
+        # returns: self
+
         def on_click_show_preview(self):
             self._onclick_showpreview_html = True
             self.on_click("quykHtml_showPreview(this);")
@@ -701,6 +785,10 @@ class qhtml:
             self.add_attribute('onmouseout="' + _code + '"')
             return self
 
+        # Gets ajax data if any and retuns a list
+        # of data if no specified type is passed
+        # returns: data/boolean
+
         def ajax_get(self, _type=""):
             if self.ajax_code != "":
                 if _type == "":
@@ -713,10 +801,12 @@ class qhtml:
                     return self.ajax_callback
             return False
 
-        # previous build -> _build(self, _type, _str_path, _js_func_name_and_callback_func: list, _async="true"):
+        # Attempts to build an ajax request. randomize_call_for.. is used
+        # to get the most recent data from the request by providing
+        # a piece of raw js code that appends a random number to the call request
+        # returns: self
 
         def ajax_build(self, _type, _str_path, js_callback_func="", _async="true", randomize_call_for_fresh_data=False):
-            # if isinstance(_js_func_name_and_callback_func, list):
             _ran = []
             for i in range(5):
                 _ran.append(str(random.randint(0, 9)))
@@ -732,12 +822,9 @@ class qhtml:
                 if "(" not in _callback_name:
                     _callback_name = _callback_name + "(r.responseText)"
             else:
-                # print("error in ajax_build_html(...) - > _js_func_name_and_callback_func should be a list with 2 entries [x,y]. \nX being the function for the ajax to be called and Y being the callback function formatted like 'callback_function(r.responseText)'.")
                 print("ajax_build FAILED:\nPlease provide a Javascript function name and a Javascript callback method to handle the response text.\nAs js_func_name and js_callback_func")
                 return self
 
-            # _func_name = "_ajax_handler_" + str(len(self.ajax_list))
-
             _r = 'function ' + _func_name + '{var r = new XMLHttpRequest();'
             _r = _r + 'r.onreadystatechange = function () {'
             _r = _r + 'if (r.readyState == 4 && r.status == 200) {'
@@ -746,6 +833,7 @@ class qhtml:
             _r = _r + '     var a = "";'
             _r = _r + '  }'
             _r = _r + '};'
+
             if randomize_call_for_fresh_data:
                 _r = _r + 'r.open("' + _type + '", "' + _str_path + '?ran=" + Math.random().toString().slice(2),' + _async + ');'
             else:
@@ -758,22 +846,12 @@ class qhtml:
 
             return self
 
-            # print("ajax code - > " + _r)
-
-            # return _func_name + ";"
+        # Returns true if the element has an on click preview
+        # returns: self
 
         def has_preview(self):
             return self._onclick_showpreview_html
 
-        def set_img_placeholder(self, place_holder_size=150):
-            if self.type != 'img':
-                print('qhtml object set_img_placeholder error.\nShould be used on img type, you used it on ' + self.type)
-                return False
-            self.set_img_src('https://via.placeholder.com/' + str(place_holder_size))
-            return self
-
-        # CLASS style object
-
         class style_obj:
 
             # INITIALIZE
@@ -800,45 +878,72 @@ class qhtml:
 
                 return self.parent
 
+            # Appends inline styling to an elements style class
+            # returns: style.parent (element)
+
             def append(self, _style):
                 self._style = self._style + _style
                 return self.parent
 
+            # Helper to align an element
+            # returns: style.parent (element)
+
             def align(self, left_center_right="center"):
                 self.append('text-align:' + left_center_right + ';')
                 # self._style = self._style + 'text-align:' + left_center_right + ';'
                 return self.parent
 
+            # Background color helper
+            # returns: style.parent (element)
+
             def bg_color(self, color='white'):
                 self.append('background-color:' + color + ';')
                 # self._style = self._style + 'background-color:' + color + ';'
                 return self.parent
 
+            # Float helper for an element
+            # returns: style.parent (element)
+
             def float(self, left_or_right="left"):
                 self.append('text-align:' + left_or_right + ';')
                 # self._style = self._style + 'text-align:' + left_or_right + ';'
                 return self.parent
 
+            # Font size helper method
+            # returns: style.parent (element)
+
             def font_size(self, size="18px"):
                 self.append('font-size:' + size + ';')
                 # self._style = self._style + 'font-size:' + size + ';'
                 return self.parent
 
+            # Font color helper method
+            # returns: style.parent (element)
+
             def font_color(self, color='black'):
                 self.append('color:' + color + ';')
                 # self._style += 'color:' + color + ';'
                 return self.parent
 
+            # Height helper method
+            # returns: style.parent (element)
+
             def height(self, height):
                 self.append('height:' + height + ';')
                 # self._style += 'height:' + height + ';'
                 return self.parent
 
+            # Width helper method
+            # returns: style.parent (element)
+
             def width(self, width):
                 self.append('width:' + width + ';')
                 # self._style += 'width:' + width + ';'
                 return self.parent
 
+            # Hide helper method
+            # returns: style.parent (element)
+
             def hide(self, none_or_hidden='none'):
                 self.append('display:' + none_or_hidden + ';')
                 # self._style += 'display:' + none_or_hidden + ';'
@@ -851,6 +956,7 @@ class qhtml:
             self.td_styles = []
             self.td_classes = []
             self.td_ids = []
+            self.objects = []
 
             if type(rows_or_file_path) != int:
                 _styling = columns_or_styling_dict
@@ -895,12 +1001,19 @@ class qhtml:
                                     # here
                                     _styling_value = _styling[_p.type]
                                     _values_dict = _styling[_p.type]
-                                    _calls = _values_dict["calls"]
-                                    _styling_value = _styling_value["style"]
-                                    _p.style.set(_styling_value)
+                                    if "calls" in _values_dict:
+                                        _calls = _values_dict["calls"]
+                                    else:
+                                        _calls = []
+
+                                    if "style" in _styling_value:
+                                        _styling_value = _styling_value["style"]
+                                        _p.style.set(_styling_value)
+                                    else:
+                                        _styling_value = []
 
                                     for __call in _calls:
-                                        print("call - > " + str(__call))
+                                        # print("call - > " + str(__call))
                                         # call method
                                         __call_m = __call[0]
                                         # call arg ( may need to add support for multiple args )
@@ -912,8 +1025,8 @@ class qhtml:
                                             print("call - > " + str(__call) + "\n return - >  " + str(_val))
 
                             _table.insert_at(_curr_row, _curr_col, _p)
-                        _curr_col = -1
 
+                        _curr_col = -1
                     self.get = _table
 
                 else:
@@ -921,6 +1034,7 @@ class qhtml:
                     self.columns = 0
                     self.time = "Error"
                     self.error = -1
+                    self.objects = []
 
             else:
                 self.objects = []
@@ -928,9 +1042,11 @@ class qhtml:
                 self.columns = columns_or_styling_dict
                 self.time = 0
 
+        # Insert an element object into *row* and *column* using 0 based index
+        # returns: self
+
         def insert_at(self, row, column, obj):
             _s = self
-
             if type(obj) is list:
                 for li in obj:
                     li.table_inserted_at = [str(row), str(column)]
@@ -943,6 +1059,9 @@ class qhtml:
 
             return self
 
+        # Set the table td id at *row* and *column* using 0 based index
+        # returns: self
+
         def set_td_id_at(self, row, col, _id):
             _s = self
             _s.td_ids.append({
@@ -955,6 +1074,9 @@ class qhtml:
 
             return self
 
+        # Set the table td class at *row* and *column* using 0 based index
+        # returns: self
+
         def set_td_class_at(self, row, col, _class):
             _s = self
             _s.td_classes.append({
@@ -967,10 +1089,11 @@ class qhtml:
 
             return self
 
+        # Set the table td style at *row* and *column* using 0 based index
+        # returns: self
+
         def style_td_at(self, row, col, style):
             _s = self
-            # print("\nSTYLING AT " + str(row) + " - " + str(col) + '\n')
-            # _s.td_styles
             _s.td_styles.append({
                 "style": style,
                 "row": row,
@@ -981,15 +1104,18 @@ class qhtml:
 
             return self
 
+        # Attempts to build the table and then insert it into a div
+        # returns: div (table inside)
+
         def build(self, append_html=False):
             """Builds the table object into a div object and returns that div."""
-            q = qhtml()
+            q = self.__qhtml
             div = q.new("div")
             self.__build_into(div, append_html)
             return div
 
         def __build_into(self, _obj: object, append_html=False):
-            if not isinstance(_obj, qhtml.new_obj):
+            if not isinstance(_obj, qhtml._q_element):
                 print("BUILD " + str(self) + " - obj = " + str(_obj) + " - " + str(type(_obj)))
                 print("Error building table -> table.build(_qhtml_object_element) -> argument should be a qhtml.new(type) object.")
                 return False
@@ -1000,7 +1126,6 @@ class qhtml:
         def __build_html(self):
             row_index = -1
             column_index = -1
-
             html_table_open = '<table style="border-collapse: collapse;table-layout: fixed;width:100%;"><tbody ' \
                               'style="width:100%;"> '
             html_table_close = "</tbody></table>"
@@ -1051,17 +1176,22 @@ class qhtml:
                         html_mid_build = html_mid_build + "<td>"
 
                     _td_id_set = ""
+                    try:
+                        ob = self.objects
+                        if len(self.objects) == 0:
+                            ob = self.get.objects
+                        for o in ob:
+                            if o.table_inserted_at[0] == str(row_index) and o.table_inserted_at[1] == str(column_index):
 
-                    for o in self.objects:
-                        if o.table_inserted_at[0] == str(row_index) and o.table_inserted_at[1] == str(column_index):
-
-                            # INSERT OBJECT HTML INTO TABLE HERE
-                            # -----------------------------------
-                            if o.innerText not in o.innerHTML:
-                                html_mid_build += o.get_tag_open() + o.innerText + o.innerHTML + o.get_tag_close() + ""
-                            else:
-                                html_mid_build += o.get_tag_open() + o.innerHTML + o.get_tag_close() + ""
-                            # -----------------------------------
+                                # INSERT OBJECT HTML INTO TABLE HERE
+                                # -----------------------------------
+                                if o.innerText not in o.innerHTML:
+                                    html_mid_build += o.get_tag_open() + o.innerText + o.innerHTML + o.get_tag_close() + ""
+                                else:
+                                    html_mid_build += o.get_tag_open() + o.innerHTML + o.get_tag_close() + ""
+                                # -----------------------------------
+                    except Exception as e:
+                        print(e)
 
                     html_mid_build += "</td>"
 
@@ -1070,7 +1200,6 @@ class qhtml:
                 column_index = -1
 
             if not html_mid_build == "":
-                # print(html_table_open + html_mid_build + html_table_close)
                 self.time = int(round(time.time() * 1000)) - self.time_start
                 print("\n** Table built in " + str(self.time) + " MS with " + str(self.columns) + " columns and " + str(self.rows) + " rows  ** \n")
                 return html_table_open + html_mid_build + html_table_close
@@ -1081,12 +1210,20 @@ class qhtml:
         def __init__(self):
             self._using = False
 
+        # Set using bootstrap true or false
+        # returns: void
+
         def use(self, _bool):
             self._using = _bool
 
+        # Returns true if using bootstrap
+        # returns: Boolean
+
         def using(self):
             return self._using
 
+        # Get the bootstrap raw code
+        # returns: string (bootstrap code)
         def get(self):
             _s = self
             bss = '<!-- CSS only --><link rel="stylesheet" ' \
@@ -1120,6 +1257,9 @@ class qhtml:
 
             return False
 
+        # Saves the template (list of quykhtml elements) to a pickle object
+        # returns: boolean
+
         def save(self, template_name: str, qhtml_obj_list: list):
             s = self
             if template_name:
@@ -1137,6 +1277,9 @@ class qhtml:
 
             return True
 
+        # Attempts to load the FOLDER containing all or one pickle object
+        # returns: list (of quykhtml elements)
+
         def load(self, template_name):
             s = self
             if template_name:
@@ -1156,3 +1299,73 @@ class qhtml:
                     return _templates
 
             return False
+
+    class __seo:
+        def __init__(self, qhtml_parent):
+            self.__title = ""
+            self.__description = ""
+            self.parent = qhtml_parent  # type: qhtml
+
+        def display_all_seo(self):
+            print(str(len(self.parent.head)) + ' SEO Head Tags - > ' + str(self.parent.head))
+            return self.parent.head
+
+        def set_page_title(self, _str):
+            if _str:
+                self.parent.head.append('<title>' + _str + '<title>')
+
+        def set_page_description(self, _str):
+            if _str:
+                self.parent.head.append('<meta name="description" content="' + _str + '">')
+
+        def set_page_keywords(self, _str):
+            if _str:
+                self.parent.head.append('<meta name="keywords" content="' + _str + '">')
+
+        def set_page_author(self, _str):
+            if _str:
+                self.parent.head.append('<meta name="author" content="' + _str + '">')
+
+        def set_page_viewport(self, _str):
+            if _str:
+                self.parent.head.append('<meta name="author" content="' + _str + '">')
+
+        def set_page_auto_refresh(self, seconds: int):
+            if seconds:
+                self.parent.head.append('<meta http-equiv="refresh" content="' + str(seconds) + '">')
+
+        def set_page_robots(self, _str):
+            allowed = ['noindex', 'nofollow', 'index', 'follow']
+            passed = False
+            for a in allowed:
+                if a in _str:
+                    passed = True
+            if _str:
+                if passed:
+                    self.parent.head.append('<meta name="robots" content="' + _str + '">')
+                else:
+                    print('seo.set_page_robots ERROR.\nThese are the allowed strings to be passed: ' + str(allowed))
+
+        def set_page_encoding(self, encoding='UTF-8'):
+            if encoding:
+                self.parent.head.append('<meta charset="' + encoding + '">')
+
+        def set_page_subject(self, _str):
+            if _str:
+                self.parent.head.append('<meta name="subject" content="' + _str + '">')
+
+        def set_page_classification(self, _str):
+            if _str:
+                self.parent.head.append('<meta name="Classification" content="' + _str + '">')
+
+        def set_page_designer(self, _str):
+            if _str:
+                self.parent.head.append('<meta name="designer" content="' + _str + '">')
+
+        def set_page_copyright(self, _str):
+            if _str:
+                self.parent.head.append('<meta name="copyright" content="' + _str + '">')
+
+        def set_page_category(self, _str):
+            if _str:
+                self.parent.head.append('<meta name="category" content="' + _str + '">')
